@@ -32,6 +32,7 @@
 #include <QLineEdit>
 #include <QList>
 #include <QMimeData>
+#include <QPushButton>
 #include <QStringList>
 #include <QStringLiteral>
 #include <QUrl>
@@ -85,6 +86,9 @@ private Q_SLOTS:
     void testTokenize();
     void testTokenizeForSave_data();
     void testTokenizeForSave();
+    void testThumbnailPreviewSetting();
+    void testReplaceLocationEditFilename_data();
+    void testReplaceLocationEditFilename();
 };
 
 void KFileWidgetTest::initTestCase()
@@ -445,6 +449,11 @@ void KFileWidgetTest::testExtensionForSave_data()
 
     // If the file literally exists, then no new extension will be appended.
     QTest::newRow("existing file") << "README" << filter << QStringLiteral("README") << QStringLiteral("README");
+
+    // test bug 382437
+    const QString octetStreamfilter = QStringLiteral("application/octet-stream");
+    QTest::newRow("octetstream.noext") << "some" << octetStreamfilter << QStringLiteral("some") << QStringLiteral("some");
+    QTest::newRow("octetstream.ext") << "some.txt" << octetStreamfilter << QStringLiteral("some.txt") << QStringLiteral("some.txt");
 }
 
 void KFileWidgetTest::testExtensionForSave()
@@ -668,6 +677,8 @@ void KFileWidgetTest::testTokenize_data()
     // file, the whole name will not be escaped.
     QTest::newRow("quote-single-file") << QStringList{"test\"quote"} << QString("test\\\"quote");
 
+    QTest::newRow("single-file-with-two-quotes") << QStringList{"\"test\".txt"} << QString{"\\\"test\\\".txt"};
+
     // Escape multiple files. Files should also be wrapped in ""
     // Note that we are also testing quote at the end of the name
     QTest::newRow("quote-multi-file") << QStringList{"test\"quote", "test2-quote\"", "test"} << QString("\"test\\\"quote\" \"test2-quote\\\"\" \"test\"");
@@ -788,6 +799,7 @@ void KFileWidgetTest::testTokenizeForSave_data()
     // All quotes in names should be escaped, however since this is a single
     // file, the whole name will not be escaped.
     QTest::newRow("save-quote") << QString{"test\"quote"} << QString() << QString("test\\\"quote");
+    QTest::newRow("save-file-with-quotes") << QString{"\"test\".txt"} << QString() << QString{"\\\"test\\\".txt"};
 
     // Ok, enough with quotes... lets do some backslashes
     // Backslash literals in file names - Unix only case
@@ -875,6 +887,136 @@ void KFileWidgetTest::testTokenizeForSave()
     QCOMPARE(urls[0], fileUrl);
 }
 
+// BUG: 501743
+void KFileWidgetTest::testThumbnailPreviewSetting()
+{
+    QTemporaryDir tempDir;
+    QUrl path = QUrl::fromLocalFile(tempDir.path());
+    auto getAction = [](KFileWidget *fw) {
+        QAction *previewAction = nullptr;
+        for (auto action : fw->actions()) {
+            if (action->text() == "Show Preview") {
+                previewAction = action;
+                break;
+            }
+        }
+        return previewAction;
+    };
+
+    // Set up
+    KFileWidget fwSetup(path);
+    fwSetup.setOperationMode(KFileWidget::Saving);
+    fwSetup.setMode(KFile::File);
+    QAction *previewAction = getAction(&fwSetup);
+    QVERIFY(previewAction);
+    previewAction->setChecked(true);
+    QCOMPARE(previewAction->isChecked(), true);
+    fwSetup.cancelButton()->click();
+
+    // Check preview settings are true, then save them as false
+    KFileWidget fwPreviewTrue(path);
+    fwPreviewTrue.setOperationMode(KFileWidget::Saving);
+    fwPreviewTrue.setMode(KFile::File);
+    previewAction = getAction(&fwPreviewTrue);
+    QVERIFY(previewAction);
+    QCOMPARE(previewAction->isChecked(), true);
+    previewAction->setChecked(false);
+    QCOMPARE(previewAction->isChecked(), false);
+    fwPreviewTrue.cancelButton()->click();
+
+    // Check preview settings are saved as false
+    KFileWidget fwPreviewFalse(path);
+    fwPreviewFalse.setOperationMode(KFileWidget::Saving);
+    fwPreviewFalse.setMode(KFile::File);
+    previewAction = getAction(&fwPreviewFalse);
+    QVERIFY(previewAction);
+    QCOMPARE(previewAction->isChecked(), true);
+    fwPreviewFalse.cancelButton()->click();
+}
+
+struct LocationTestItem {
+    bool dir;
+    QString name;
+};
+
+void KFileWidgetTest::testReplaceLocationEditFilename_data()
+{
+    QTest::addColumn<LocationTestItem>("initialItem");
+    QTest::addColumn<LocationTestItem>("selectedItem");
+    QTest::addColumn<QString>("lineEditTextResult");
+    QTest::addColumn<bool>("overrideModifiedText");
+
+    QTest::newRow("replace-dir-with-dir") << LocationTestItem(true, "folder1") << LocationTestItem(true, "folder2") << "" << false;
+    QTest::newRow("replace-dir-with-file") << LocationTestItem(true, "folder1") << LocationTestItem(false, "file1") << "file1" << true;
+    QTest::newRow("replace-file-with-file") << LocationTestItem(false, "file1") << LocationTestItem(false, "file2") << "file2" << true;
+    QTest::newRow("replace-file-with-dir") << LocationTestItem(false, "file1") << LocationTestItem(true, "folder1") << "file1" << false;
+}
+
+// BUG: 502794
+// Test that we don't override file names with folder names
+void KFileWidgetTest::testReplaceLocationEditFilename()
+{
+    QFETCH(LocationTestItem, initialItem);
+    QFETCH(LocationTestItem, selectedItem);
+    QFETCH(QString, lineEditTextResult);
+    QFETCH(bool, overrideModifiedText);
+
+    // Setup - Create folders/files in temp dir
+    QTemporaryDir tempDir;
+    const QString tempDirPath = tempDir.path();
+    QUrl tempDirUrl = QUrl::fromLocalFile(tempDirPath);
+    QUrl replacedUrl = QUrl::fromLocalFile(tempDirPath + QLatin1Char('/') + initialItem.name);
+    QUrl selectedUrl = QUrl::fromLocalFile(tempDirPath + QLatin1Char('/') + selectedItem.name);
+
+    auto createTestItem = [tempDirUrl](LocationTestItem item, const QUrl &url) {
+        if (item.dir) {
+            QDir(tempDirUrl.toLocalFile()).mkdir(url.toLocalFile());
+            QVERIFY(QDir(url.toLocalFile()).exists());
+        } else {
+            QFile file(url.toLocalFile());
+            if (!file.open(QIODevice::WriteOnly)) {
+                qFatal("Couldn't create %s", qPrintable(url.toLocalFile()));
+            }
+            file.write(QByteArray("Test file"));
+            file.close();
+            QVERIFY(file.exists());
+        }
+    };
+
+    createTestItem(initialItem, replacedUrl);
+    createTestItem(selectedItem, selectedUrl);
+
+    // Open the filewidget in tempdir
+    KFileWidget fw(tempDirUrl);
+    fw.setOperationMode(KFileWidget::Saving);
+
+    // Highlight the item, then another
+    auto highlightItem = [&fw](QUrl url) {
+        KFileItem fileItem(url);
+        QSignalSpy fileHighlightedSpy(fw.dirOperator(), &KDirOperator::fileHighlighted);
+        fw.dirOperator()->highlightFile(fileItem);
+        fileHighlightedSpy.wait(500);
+        QVERIFY(fileHighlightedSpy.count());
+    };
+
+    highlightItem(replacedUrl);
+    highlightItem(selectedUrl);
+
+    // Compare that we have the wanted result when selecting items
+    QCOMPARE(fw.locationEdit()->lineEdit()->text(), lineEditTextResult);
+
+    // Make sure we don't overwrite any text user has modified in some cases
+    const QString modifiedText("New Filename.txt");
+    fw.locationEdit()->setEditText(modifiedText);
+    highlightItem(selectedUrl);
+
+    if (overrideModifiedText) {
+        QCOMPARE(fw.locationEdit()->lineEdit()->text(), lineEditTextResult);
+    } else {
+        QCOMPARE(fw.locationEdit()->lineEdit()->text(), modifiedText);
+    }
+}
+
 QTEST_MAIN(KFileWidgetTest)
 
 #include "kfilewidgettest.moc"
