@@ -8,6 +8,7 @@
 #ifndef KDIROPERATORICONVIEW_P_H
 #define KDIROPERATORICONVIEW_P_H
 
+#include <KDirOperator>
 #include <QListView>
 
 /**
@@ -18,22 +19,28 @@ class KDirOperatorIconView : public QListView
 {
     Q_OBJECT
 public:
-    KDirOperatorIconView(QWidget *parent = nullptr, QStyleOptionViewItem::Position decorationPosition = QStyleOptionViewItem::Position::Top);
-    virtual ~KDirOperatorIconView();
+    KDirOperatorIconView(KDirOperator *dirOperator,
+                         QWidget *parent = nullptr,
+                         QStyleOptionViewItem::Position decorationPosition = QStyleOptionViewItem::Position::Top);
+    ~KDirOperatorIconView() override;
     void setDecorationPosition(QStyleOptionViewItem::Position decorationPosition);
 
 protected:
-    QStyleOptionViewItem viewOptions() const override;
+    void initViewItemOption(QStyleOptionViewItem *option) const override;
     void dragEnterEvent(QDragEnterEvent *event) override;
     void mousePressEvent(QMouseEvent *event) override;
+    void mouseMoveEvent(QMouseEvent *event) override;
+    void mouseReleaseEvent(QMouseEvent *event) override;
     void wheelEvent(QWheelEvent *event) override;
     void resizeEvent(QResizeEvent *event) override;
 
-protected Q_SLOT:
-    void updateLayout();
+protected
+    Q_SLOT : void updateLayout();
+
 private:
     QStyleOptionViewItem::Position decorationPosition;
+    bool m_isEmblemClicked;
+    KDirOperator *m_dirOperator;
 };
 
-
 #endif // KDIROPERATORICONVIEW_P_H
