@@ -60,7 +60,6 @@
 using namespace std::chrono_literals;
 
 static constexpr int s_lateralMargin = 4;
-static constexpr int s_capacitybarHeight = 6;
 static constexpr auto s_pollFreeSpaceInterval = 1min;
 
 KFilePlacesViewDelegate::KFilePlacesViewDelegate(KFilePlacesView *parent)
@@ -1181,7 +1180,7 @@ void KFilePlacesView::contextMenuEvent(QContextMenuEvent *event)
 
     if (!clickOverEmptyArea) {
         if (placeUrl.scheme() == QLatin1String("trash")) {
-            emptyTrash = new QAction(QIcon::fromTheme(QStringLiteral("trash-empty")), i18nc("@action:inmenu", "Empty Trash"), &menu);
+            emptyTrash = new QAction(QIcon::fromTheme(QStringLiteral("edit-delete")), i18nc("@action:inmenu", "Empty Trash"), &menu);
             KConfig trashConfig(QStringLiteral("trashrc"), KConfig::SimpleConfig);
             emptyTrash->setEnabled(!trashConfig.group(QStringLiteral("Status")).readEntry("Empty", true));
         }
@@ -1928,8 +1927,8 @@ void KFilePlacesViewPrivate::addPlace(const QModelIndex &index)
 
     QUrl url = m_currentUrl;
     QString label;
-    QString iconName = QStringLiteral("folder");
-    bool appLocal = true;
+    QString iconName = QString(); // let it figure out the icon for itself
+    bool appLocal = false;
     if (KFilePlaceEditDialog::getInformation(true, url, label, iconName, true, appLocal, 64, q)) {
         QString appName;
         if (appLocal) {
