@@ -38,6 +38,8 @@
 #include <QMetaMethod>
 #include <QMimeData>
 #include <QMimeDatabase>
+#include <QPainter>
+#include <QStyle>
 #include <QTimer>
 #include <QUrlQuery>
 
@@ -197,6 +199,9 @@ public:
     bool m_active = true;
     bool m_showPlacesSelector = false;
     bool m_showFullPath = false;
+    bool m_backgroundEnabled = true;
+
+    int m_padding = 5;
 
     struct {
         bool showHidden = false;
@@ -211,6 +216,8 @@ KUrlNavigatorPrivate::KUrlNavigatorPrivate(const QUrl &url, KUrlNavigator *qq, K
 {
     m_layout->setSpacing(0);
     m_layout->setContentsMargins(0, 0, 0, 0);
+    QStyleOption option;
+    option.initFrom(q);
 
     q->connect(m_coreUrlNavigator, &KCoreUrlNavigator::currentLocationUrlChanged, q, [this]() {
         Q_EMIT q->urlChanged(m_coreUrlNavigator->currentLocationUrl());
@@ -265,6 +272,9 @@ KUrlNavigatorPrivate::KUrlNavigatorPrivate(const QUrl &url, KUrlNavigator *qq, K
     m_pathBox = new KUrlComboBox(KUrlComboBox::Directories, true, q);
     m_pathBox->setSizeAdjustPolicy(QComboBox::AdjustToContentsOnFirstShow);
     m_pathBox->installEventFilter(q);
+    m_pathBox->setAutoFillBackground(false);
+    m_pathBox->setBackgroundRole(QPalette::Base);
+    m_pathBox->setFrame(false);
 
     KUrlCompletion *kurlCompletion = new KUrlCompletion(KUrlCompletion::DirCompletion);
     m_pathBox->setCompletionObject(kurlCompletion);
@@ -299,12 +309,20 @@ KUrlNavigatorPrivate::KUrlNavigatorPrivate(const QUrl &url, KUrlNavigator *qq, K
     m_layout->addWidget(m_dropDownButton);
     m_layout->addWidget(m_pathBox, 1);
     m_layout->addWidget(m_badgeWidgetContainer);
+    m_layout->addSpacing(q->style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing, &option, q));
     m_layout->addWidget(m_toggleEditableMode);
 
     q->setContextMenuPolicy(Qt::CustomContextMenu);
     q->connect(q, &QWidget::customContextMenuRequested, q, [this](const QPoint &pos) {
         openContextMenu(pos);
     });
+
+    // Make sure pathBox does not portrude outside of the above frameLineEdit background
+    const int paddingLeft = q->style()->pixelMetric(QStyle::PM_LayoutLeftMargin);
+    const int paddingRight = q->style()->pixelMetric(QStyle::PM_LayoutRightMargin);
+    q->rect().adjust(0, -1, 0, 1);
+    q->setContentsMargins(paddingLeft, 1, paddingRight, 1);
+    m_pathBox->setContentsMargins(paddingLeft, 0, paddingRight, 0);
 }
 
 void KUrlNavigatorPrivate::appendWidget(QWidget *widget, int stretch)
@@ -537,11 +555,14 @@ void KUrlNavigatorPrivate::switchView()
     m_toggleEditableMode->setChecked(m_editable);
     updateContent();
     if (q->isUrlEditable()) {
+        m_pathBox->setFixedHeight(m_badgeWidgetContainer->height());
         m_pathBox->setFocus();
     }
 
     q->requestActivation();
     Q_EMIT q->editableStateChanged(m_editable);
+    // Make sure the colors are updated
+    q->update();
 }
 
 void KUrlNavigatorPrivate::dropUrls(const QUrl &destination, QDropEvent *event, KUrlNavigatorButton *dropButton)
@@ -812,6 +833,7 @@ void KUrlNavigatorPrivate::updateButtonVisibility()
     }
 
     // Subtract all widgets from the available width, that must be shown anyway
+    // Make sure to take the padding into account
     int availableWidth = q->width() - m_toggleEditableMode->minimumWidth();
 
     availableWidth -= m_badgeWidgetContainer->width();
@@ -824,18 +846,10 @@ void KUrlNavigatorPrivate::updateButtonVisibility()
         availableWidth -= m_schemes->width();
     }
 
-    // Check whether buttons must be hidden at all...
-    int requiredButtonWidth = 0;
-    for (const auto *button : std::as_const(m_navButtons)) {
-        requiredButtonWidth += button->minimumWidth();
-    }
+    availableWidth -= m_dropDownButton->width();
 
-    if (requiredButtonWidth > availableWidth) {
-        // At least one button must be hidden. This implies that the
-        // drop-down button must get visible, which again decreases the
-        // available width.
-        availableWidth -= m_dropDownButton->width();
-    }
+    // Count the paddings of previous button and current button
+    availableWidth -= m_padding * 4;
 
     // Hide buttons...
     bool isLastButton = true;
@@ -875,13 +889,22 @@ void KUrlNavigatorPrivate::updateButtonVisibility()
         m_dropDownButton->setVisible(visible);
     }
 
+    auto lastButton = m_navButtons.last();
+    for (const auto &button : m_navButtons) {
+        if (button != lastButton) {
+            button->setDrawSeparator(true);
+        } else {
+            button->setDrawSeparator(false);
+        }
+    }
+
     updateTabOrder();
 }
 
 void KUrlNavigatorPrivate::updateTabOrder()
 {
     QMultiMap<int, QWidget *> visibleChildrenSortedByX;
-    const auto childWidgets = q->findChildren<QWidget *>();
+    const auto childWidgets = q->findChildren<KUrlNavigatorButtonBase *>();
     for (auto childWidget : childWidgets) {
         if (childWidget->isVisible()) {
             if (q->layoutDirection() == Qt::LeftToRight) {
@@ -891,6 +914,7 @@ void KUrlNavigatorPrivate::updateTabOrder()
             }
         }
     }
+
     if (visibleChildrenSortedByX.isEmpty()) {
         return;
     }
@@ -902,6 +926,7 @@ void KUrlNavigatorPrivate::updateTabOrder()
         it++;
         nextIt++;
     }
+    Q_EMIT q->layoutChanged();
 }
 
 QString KUrlNavigatorPrivate::firstButtonText() const
@@ -1009,7 +1034,9 @@ KUrlNavigator::KUrlNavigator(KFilePlacesModel *placesModel, const QUrl &url, QWi
 
     setMinimumWidth(100);
 
+    installEventFilter(this);
     d->updateContent();
+    d->updateTabOrder();
 }
 
 KUrlNavigator::~KUrlNavigator()
@@ -1019,6 +1046,7 @@ KUrlNavigator::~KUrlNavigator()
     for (auto *button : std::as_const(d->m_navButtons)) {
         button->removeEventFilter(this);
     }
+    removeEventFilter(this);
 }
 
 QUrl KUrlNavigator::locationUrl(int historyIndex) const
@@ -1132,8 +1160,11 @@ void KUrlNavigator::setPlacesSelectorVisible(bool visible)
     }
 
     d->m_showPlacesSelector = visible;
-    d->m_placesSelector->setVisible(visible);
-    d->updateTabOrder();
+
+    if (d->m_placesSelector) {
+        d->m_placesSelector->setVisible(visible);
+        d->updateTabOrder();
+    }
 }
 
 bool KUrlNavigator::isPlacesSelectorVisible() const
@@ -1206,10 +1237,18 @@ void KUrlNavigator::mouseReleaseEvent(QMouseEvent *event)
             // toggle-editable-mode-button. Paste the clipboard content
             // as location URL.
             QClipboard *clipboard = QApplication::clipboard();
-            const QMimeData *mimeData = clipboard->mimeData();
-            if (mimeData->hasText()) {
+            const QMimeData *mimeData = clipboard->mimeData(QClipboard::Mode::Selection);
+            if (mimeData && mimeData->hasText()) {
                 const QString text = mimeData->text();
-                setLocationUrl(QUrl::fromUserInput(text));
+                const auto currentUrl = d->m_coreUrlNavigator->currentLocationUrl();
+                QString workindDirectory;
+                if (currentUrl.isLocalFile()) {
+                    workindDirectory = currentUrl.toLocalFile();
+                }
+                auto url = QUrl::fromUserInput(text, workindDirectory);
+                if (url.isValid()) {
+                    setLocationUrl(url);
+                }
             }
         }
     }
@@ -1230,6 +1269,12 @@ void KUrlNavigator::wheelEvent(QWheelEvent *event)
     QWidget::wheelEvent(event);
 }
 
+void KUrlNavigator::showEvent(QShowEvent *event)
+{
+    d->updateTabOrder();
+    QWidget::showEvent(event);
+}
+
 bool KUrlNavigator::eventFilter(QObject *watched, QEvent *event)
 {
     switch (event->type()) {
@@ -1241,12 +1286,14 @@ bool KUrlNavigator::eventFilter(QObject *watched, QEvent *event)
         for (KUrlNavigatorButton *button : std::as_const(d->m_navButtons)) {
             button->setShowMnemonic(true);
         }
+        update();
         break;
 
     case QEvent::FocusOut:
         for (KUrlNavigatorButton *button : std::as_const(d->m_navButtons)) {
             button->setShowMnemonic(false);
         }
+        update();
         break;
 
     // Avoid the "Properties" action from triggering instead of new tab.
@@ -1260,6 +1307,25 @@ bool KUrlNavigator::eventFilter(QObject *watched, QEvent *event)
         break;
     }
 
+#if KIO_VERSION < QT_VERSION_CHECK(7, 0, 0)
+    case QEvent::Paint: {
+        // We can't call this in overridden paintEvent since applications using
+        // the paint event is handled through the event filter:
+        // Overriding paintEvent might not have an effect in applications
+        // compiled against the older KIO, as they might work with an older vtable.
+        // However, they would still see the new button style.
+        // This makes sure the background is always drawn.
+        if (watched == this) {
+            auto *pEvent = static_cast<QPaintEvent *>(event);
+            if (pEvent) {
+                KUrlNavigator::paintEvent(pEvent);
+                return true;
+            }
+        }
+        break;
+    }
+#endif
+
     default:
         break;
     }
@@ -1342,4 +1408,40 @@ QWidget *KUrlNavigator::badgeWidget() const
     }
 }
 
+void KUrlNavigator::setBackgroundEnabled(bool enabled)
+{
+    d->m_backgroundEnabled = enabled;
+}
+
+bool KUrlNavigator::isBackgroundEnabled() const
+{
+    return d->m_backgroundEnabled;
+}
+
+void KUrlNavigator::paintEvent(QPaintEvent *event)
+{
+    Q_UNUSED(event);
+    QPainter painter(this);
+    QStyleOptionFrame option;
+    option.initFrom(this);
+    option.state = QStyle::State_None;
+
+    if (hasFocus()) {
+        option.palette.setColor(QPalette::Window, palette().color(QPalette::Highlight));
+    }
+
+    if (d->m_backgroundEnabled) {
+        // Draw primitive always, but change color if not editable
+        if (!d->m_editable) {
+            option.palette.setColor(QPalette::Base, palette().alternateBase().color());
+        }
+        style()->drawPrimitive(QStyle::PE_FrameLineEdit, &option, &painter, this);
+    } else {
+        // Draw primitive only for the input field
+        if (d->m_editable) {
+            style()->drawPrimitive(QStyle::PE_FrameLineEdit, &option, &painter, this);
+        }
+    }
+}
+
 #include "moc_kurlnavigator.cpp"
