@@ -332,6 +332,21 @@ public:
      */
     QWidget *badgeWidget() const;
 
+    /**
+     * Sets the background painting enabled or disabled for the buttons layout.
+     * In frameless styles, its recommended to set the background to disabled.
+     * Does not affect the input mode.
+     * @since 6.14
+     */
+    void setBackgroundEnabled(bool enabled);
+
+    /**
+     * Returns true if the background of the buttons layout is being painted.
+     * Does not represent the input mode background.
+     * @since 6.14
+     */
+    bool isBackgroundEnabled() const;
+
 public Q_SLOTS:
     /**
      * Sets the location to \a url. The old URL is added to the history.
@@ -438,6 +453,14 @@ Q_SIGNALS:
      */
     void urlSelectionRequested(const QUrl &url);
 
+    /**
+     * The internal layout and graphical representation of components has changed,
+     * either after an url change or after a switch between editable mode and
+     * breadcrumb mode
+     * @since 6.11
+     */
+    void layoutChanged();
+
 protected:
 #if !defined(K_DOXYGEN)
     /**
@@ -468,7 +491,11 @@ protected:
 
     void wheelEvent(QWheelEvent *event) override;
 
+    void showEvent(QShowEvent *event) override;
+
     bool eventFilter(QObject *watched, QEvent *event) override;
+
+    void paintEvent(QPaintEvent *event) override;
 #endif
 
 private:
