@@ -29,15 +29,18 @@ QPointer<KUrlNavigatorMenu> KUrlNavigatorButton::m_subDirsMenu;
 
 KUrlNavigatorButton::KUrlNavigatorButton(const QUrl &url, KUrlNavigator *parent)
     : KUrlNavigatorButtonBase(parent)
-    , m_hoverArrow(false)
+    , m_hoverOverArrow(false)
+    , m_hoverOverButton(false)
     , m_pendingTextChange(false)
     , m_replaceButton(false)
     , m_showMnemonic(false)
+    , m_drawSeparator(true)
     , m_wheelSteps(0)
     , m_url(url)
     , m_subDir()
     , m_openSubDirsTimer(nullptr)
     , m_subDirsJob(nullptr)
+    , m_padding(5)
 {
     setAcceptDrops(true);
     setUrl(url);
@@ -83,6 +86,7 @@ void KUrlNavigatorButton::setUrl(const QUrl &url)
     } else {
         setText(m_url.fileName().replace(QLatin1Char('&'), QLatin1String("&&")));
     }
+    setIcon(QIcon::fromTheme(KIO::iconNameForUrl(url)));
 }
 
 QUrl KUrlNavigatorButton::url() const
@@ -125,9 +129,10 @@ QSize KUrlNavigatorButton::sizeHint() const
 {
     QFont adjustedFont(font());
     adjustedFont.setBold(m_subDir.isEmpty());
-    // the minimum size is textWidth + arrowWidth() + 2 * BorderWidth; for the
-    // preferred size we add the BorderWidth 2 times again for having an uncluttered look
-    const int width = QFontMetrics(adjustedFont).size(Qt::TextSingleLine, plainText()).width() + arrowWidth() + 4 * BorderWidth;
+    // preferred width is textWidth, iconWidth and padding combined
+    // add extra padding in end to make sure the space between divider and button is consistent
+    // the first padding is used between icon and text, second in the end of text
+    const int width = m_padding + textWidth() + arrowWidth() + m_padding;
     return QSize(width, KUrlNavigatorButtonBase::sizeHint().height());
 }
 
@@ -144,6 +149,19 @@ bool KUrlNavigatorButton::showMnemonic() const
     return m_showMnemonic;
 }
 
+void KUrlNavigatorButton::setDrawSeparator(bool draw)
+{
+    if (m_drawSeparator != draw) {
+        m_drawSeparator = draw;
+        update();
+    }
+}
+
+bool KUrlNavigatorButton::drawSeparator() const
+{
+    return m_drawSeparator;
+}
+
 void KUrlNavigatorButton::paintEvent(QPaintEvent *event)
 {
     Q_UNUSED(event);
@@ -155,6 +173,8 @@ void KUrlNavigatorButton::paintEvent(QPaintEvent *event)
     painter.setFont(adjustedFont);
 
     int buttonWidth = width();
+    int arrowWidth = KUrlNavigatorButton::arrowWidth();
+
     int preferredWidth = sizeHint().width();
     if (preferredWidth < minimumWidth()) {
         preferredWidth = minimumWidth();
@@ -163,65 +183,34 @@ void KUrlNavigatorButton::paintEvent(QPaintEvent *event)
         buttonWidth = preferredWidth;
     }
     const int buttonHeight = height();
-
     const QColor fgColor = foregroundColor();
-    drawHoverBackground(&painter);
-
-    int textLeft = 0;
-    int textWidth = buttonWidth;
-
     const bool leftToRight = (layoutDirection() == Qt::LeftToRight);
 
-    if (!m_subDir.isEmpty()) {
-        // draw arrow
-        const int arrowSize = arrowWidth();
-        const int arrowX = leftToRight ? (buttonWidth - arrowSize) - BorderWidth : BorderWidth;
-        const int arrowY = (buttonHeight - arrowSize) / 2;
-
-        QStyleOption option;
-        option.initFrom(this);
-        option.rect = QRect(arrowX, arrowY, arrowSize, arrowSize);
-        option.palette = palette();
-        option.palette.setColor(QPalette::Text, fgColor);
-        option.palette.setColor(QPalette::WindowText, fgColor);
-        option.palette.setColor(QPalette::ButtonText, fgColor);
-
-        if (m_hoverArrow) {
-            // highlight the background of the arrow to indicate that the directories
-            // popup can be opened by a mouse click
-            QColor hoverColor = palette().color(QPalette::HighlightedText);
-            hoverColor.setAlpha(96);
-            painter.setPen(Qt::NoPen);
-            painter.setBrush(hoverColor);
-
-            int hoverX = arrowX;
-            if (!leftToRight) {
-                hoverX -= BorderWidth;
-            }
-            painter.drawRect(QRect(hoverX, 0, arrowSize + BorderWidth, buttonHeight));
-        }
-
-        if (leftToRight) {
-            style()->drawPrimitive(QStyle::PE_IndicatorArrowRight, &option, &painter, this);
-        } else {
-            style()->drawPrimitive(QStyle::PE_IndicatorArrowLeft, &option, &painter, this);
-            textLeft += arrowSize + 2 * BorderWidth;
-        }
-
-        textWidth -= arrowSize + 2 * BorderWidth;
+    // Prepare sizes for icon
+    QRect textRect;
+    const int textRectWidth = buttonWidth - arrowWidth - m_padding;
+    if (leftToRight) {
+        textRect = QRect(m_padding, 0, textRectWidth, buttonHeight);
+    } else {
+        // If no separator is drawn, we can start writing text from 0
+        textRect = QRect(m_drawSeparator ? arrowWidth : 0, 0, textRectWidth, buttonHeight);
     }
 
+    drawHoverBackground(&painter);
+
+    // Draw gradient overlay if text is clipped
     painter.setPen(fgColor);
     const bool clipped = isTextClipped();
-    const QRect textRect(textLeft, 0, textWidth, buttonHeight);
     if (clipped) {
         QColor bgColor = fgColor;
         bgColor.setAlpha(0);
         QLinearGradient gradient(textRect.topLeft(), textRect.topRight());
         if (leftToRight) {
+            gradient.setFinalStop(QPoint(gradient.finalStop().x() - m_padding, gradient.finalStop().y()));
             gradient.setColorAt(0.8, fgColor);
             gradient.setColorAt(1.0, bgColor);
         } else {
+            gradient.setStart(QPoint(gradient.start().x() + m_padding, gradient.start().y()));
             gradient.setColorAt(0.0, bgColor);
             gradient.setColorAt(0.2, fgColor);
         }
@@ -231,13 +220,36 @@ void KUrlNavigatorButton::paintEvent(QPaintEvent *event)
         painter.setPen(pen);
     }
 
-    int textFlags = clipped ? Qt::AlignVCenter : Qt::AlignCenter;
+    // Draw folder name
+    int textFlags = Qt::AlignVCenter;
     if (m_showMnemonic) {
         textFlags |= Qt::TextShowMnemonic;
         painter.drawText(textRect, textFlags, text());
     } else {
         painter.drawText(textRect, textFlags, plainText());
     }
+
+    // Draw separator arrow
+    if (m_drawSeparator) {
+        QStyleOption option;
+        option.initFrom(this);
+        option.palette = palette();
+        option.palette.setColor(QPalette::Text, fgColor);
+        option.palette.setColor(QPalette::WindowText, fgColor);
+        option.palette.setColor(QPalette::ButtonText, fgColor);
+
+        if (leftToRight) {
+            option.rect = QRect(textRect.right(), 0, arrowWidth, buttonHeight);
+        } else {
+            // Separator is the first item in RtL mode
+            option.rect = QRect(0, 0, arrowWidth, buttonHeight);
+        }
+
+        if (!m_hoverOverArrow) {
+            option.state = QStyle::State_None;
+        }
+        style()->drawPrimitive(leftToRight ? QStyle::PE_IndicatorArrowRight : QStyle::PE_IndicatorArrowLeft, &option, &painter, this);
+    }
 }
 
 void KUrlNavigatorButton::enterEvent(QEnterEvent *event)
@@ -249,6 +261,10 @@ void KUrlNavigatorButton::enterEvent(QEnterEvent *event)
     if (isTextClipped()) {
         setToolTip(plainText());
     }
+    if (!m_hoverOverButton) {
+        m_hoverOverButton = true;
+        update();
+    }
 }
 
 void KUrlNavigatorButton::leaveEvent(QEvent *event)
@@ -256,8 +272,12 @@ void KUrlNavigatorButton::leaveEvent(QEvent *event)
     KUrlNavigatorButtonBase::leaveEvent(event);
     setToolTip(QString());
 
-    if (m_hoverArrow) {
-        m_hoverArrow = false;
+    if (m_hoverOverArrow) {
+        m_hoverOverArrow = false;
+        update();
+    }
+    if (m_hoverOverButton) {
+        m_hoverOverButton = false;
         update();
     }
 }
@@ -303,8 +323,9 @@ void KUrlNavigatorButton::dragEnterEvent(QDragEnterEvent *event)
 void KUrlNavigatorButton::dragMoveEvent(QDragMoveEvent *event)
 {
     QRect rect = event->answerRect();
-    if (isAboveArrow(rect.center().x())) {
-        m_hoverArrow = true;
+
+    if (isAboveSeparator(rect.center().x())) {
+        m_hoverOverArrow = true;
         update();
 
         if (m_subDirsMenu == nullptr) {
@@ -320,9 +341,11 @@ void KUrlNavigatorButton::dragMoveEvent(QDragMoveEvent *event)
         if (m_openSubDirsTimer->isActive()) {
             cancelSubDirsRequest();
         }
-        m_subDirsMenu->deleteLater();
-        m_subDirsMenu = nullptr;
-        m_hoverArrow = false;
+        if (m_subDirsMenu) {
+            m_subDirsMenu->deleteLater();
+            m_subDirsMenu = nullptr;
+        }
+        m_hoverOverArrow = false;
         update();
     }
 }
@@ -331,15 +354,15 @@ void KUrlNavigatorButton::dragLeaveEvent(QDragLeaveEvent *event)
 {
     KUrlNavigatorButtonBase::dragLeaveEvent(event);
 
-    m_hoverArrow = false;
+    m_hoverOverArrow = false;
     setDisplayHintEnabled(DraggedHint, false);
     update();
 }
 
 void KUrlNavigatorButton::mousePressEvent(QMouseEvent *event)
 {
-    if (isAboveArrow(qRound(event->position().x())) && (event->button() == Qt::LeftButton)) {
-        // the mouse is pressed above the [>] button
+    if (isAboveSeparator(qRound(event->position().x())) && (event->button() == Qt::LeftButton)) {
+        // the mouse is pressed above the folder button
         startSubDirsJob();
     }
     KUrlNavigatorButtonBase::mousePressEvent(event);
@@ -347,9 +370,9 @@ void KUrlNavigatorButton::mousePressEvent(QMouseEvent *event)
 
 void KUrlNavigatorButton::mouseReleaseEvent(QMouseEvent *event)
 {
-    if (!isAboveArrow(qRound(event->position().x())) || (event->button() != Qt::LeftButton)) {
+    if (!isAboveSeparator(qRound(event->position().x())) || (event->button() != Qt::LeftButton)) {
         // the mouse has been released above the text area and not
-        // above the [>] button
+        // above the folder button
         Q_EMIT navigatorButtonActivated(m_url, event->button(), event->modifiers());
         cancelSubDirsRequest();
     }
@@ -360,9 +383,9 @@ void KUrlNavigatorButton::mouseMoveEvent(QMouseEvent *event)
 {
     KUrlNavigatorButtonBase::mouseMoveEvent(event);
 
-    const bool hoverArrow = isAboveArrow(qRound(event->position().x()));
-    if (hoverArrow != m_hoverArrow) {
-        m_hoverArrow = hoverArrow;
+    const bool hoverOverIcon = isAboveSeparator(qRound(event->position().x()));
+    if (hoverOverIcon != m_hoverOverArrow) {
+        m_hoverOverArrow = hoverOverIcon;
         update();
     }
 }
@@ -444,10 +467,11 @@ void KUrlNavigatorButton::slotMenuActionClicked(QAction *action, Qt::MouseButton
 
 void KUrlNavigatorButton::statFinished(KJob *job)
 {
+    const KIO::UDSEntry entry = static_cast<KIO::StatJob *>(job)->statResult();
+
     if (m_pendingTextChange) {
         m_pendingTextChange = false;
 
-        const KIO::UDSEntry entry = static_cast<KIO::StatJob *>(job)->statResult();
         QString name = entry.stringValue(KIO::UDSEntry::UDS_DISPLAY_NAME);
         if (name.isEmpty()) {
             name = m_url.fileName();
@@ -456,6 +480,11 @@ void KUrlNavigatorButton::statFinished(KJob *job)
 
         Q_EMIT finishedTextResolving();
     }
+
+    const QString iconName = entry.stringValue(KIO::UDSEntry::UDS_ICON_NAME);
+    if (!iconName.isEmpty()) {
+        setIcon(QIcon::fromTheme(iconName));
+    }
 }
 
 /**
@@ -516,7 +545,7 @@ void KUrlNavigatorButton::openSubDirsMenu(KJob *job)
     initMenu(m_subDirsMenu, 0);
 
     const bool leftToRight = (layoutDirection() == Qt::LeftToRight);
-    const int popupX = leftToRight ? width() - arrowWidth() - BorderWidth : 0;
+    const int popupX = leftToRight ? width() - arrowWidth() : 0;
     const QPoint popupPos = parentWidget()->mapToGlobal(geometry().bottomLeft() + QPoint(popupX, 0));
 
     QPointer<QObject> guard(this);
@@ -630,22 +659,25 @@ int KUrlNavigatorButton::arrowWidth() const
     return width;
 }
 
-bool KUrlNavigatorButton::isAboveArrow(int x) const
+int KUrlNavigatorButton::textWidth() const
+{
+    QFont adjustedFont(font());
+    adjustedFont.setBold(m_subDir.isEmpty());
+    return QFontMetrics(adjustedFont).size(Qt::TextSingleLine, plainText()).width();
+}
+
+bool KUrlNavigatorButton::isAboveSeparator(int x) const
 {
     const bool leftToRight = (layoutDirection() == Qt::LeftToRight);
-    return leftToRight ? (x >= width() - arrowWidth()) : (x < arrowWidth());
+    return leftToRight ? (x >= width() - arrowWidth()) : (x < arrowWidth() + m_padding);
 }
 
 bool KUrlNavigatorButton::isTextClipped() const
 {
-    int availableWidth = width() - 2 * BorderWidth;
-    if (!m_subDir.isEmpty()) {
-        availableWidth -= arrowWidth() - BorderWidth;
-    }
+    // Ignore padding when resizing, so text doesnt go under it
+    int availableWidth = width() - arrowWidth() - m_padding;
 
-    QFont adjustedFont(font());
-    adjustedFont.setBold(m_subDir.isEmpty());
-    return QFontMetrics(adjustedFont).size(Qt::TextSingleLine, plainText()).width() >= availableWidth;
+    return textWidth() >= availableWidth;
 }
 
 void KUrlNavigatorButton::updateMinimumWidth()
@@ -653,8 +685,8 @@ void KUrlNavigatorButton::updateMinimumWidth()
     const int oldMinWidth = minimumWidth();
 
     int minWidth = sizeHint().width();
-    if (minWidth < 40) {
-        minWidth = 40;
+    if (minWidth < 10) {
+        minWidth = 10;
     } else if (minWidth > 150) {
         // don't let an overlong path name waste all the URL navigator space
         minWidth = 150;
