@@ -19,18 +19,29 @@ class KFileItemListProperties;
 
 namespace KIO
 {
-
 /**
-* Special flag of DropJob in addition to KIO::JobFlag
-*
-* @see DropJobFlags
-* @since 5.67
-*/
-enum DropJobFlag
-{
+ * Special flag of DropJob in addition to KIO::JobFlag
+ *
+ * @see DropJobFlags
+ * @since 5.67
+ */
+enum DropJobFlag {
     DropJobDefaultFlags = 0,
     ShowMenuManually = 1, ///< show the menu manually with DropJob::showMenu
 };
+
+/**
+ * Setting flag to determine what the default behaviour should be when dropping items.
+ *
+ * @see DndBehavior
+ * @since 6.14
+ */
+Q_NAMESPACE
+enum DndBehavior : std::uint8_t {
+    AlwaysAsk = 0,
+    MoveIfSameDevice = 1, ///< Move the dragged items without showing the options menu they are on the same device
+};
+Q_ENUM_NS(DndBehavior)
 /**
  * Stores a combination of #DropJobFlag values.
  */
@@ -56,7 +67,7 @@ class KIOWIDGETS_EXPORT DropJob : public Job
     Q_OBJECT
 
 public:
-    virtual ~DropJob();
+    ~DropJob() override;
 
     /**
      * Allows the application to set additional actions in the drop popup menu.
@@ -81,21 +92,21 @@ Q_SIGNALS:
     void itemCreated(const QUrl &url);
 
     /**
-    * Emitted when a copy job was started as subjob after user selection.
-    *
-    * You can use @p job to monitor the progress of the copy/move/link operation. Note that a
-    * CopyJob isn't always started by DropJob. For instance dropping files onto an executable will
-    * simply launch the executable.
-    *
-    * @param job the job started for moving, copying or symlinking files
-    * @since 5.30
-    */
+     * Emitted when a copy job was started as subjob after user selection.
+     *
+     * You can use @p job to monitor the progress of the copy/move/link operation. Note that a
+     * CopyJob isn't always started by DropJob. For instance dropping files onto an executable will
+     * simply launch the executable.
+     *
+     * @param job the job started for moving, copying or symlinking files
+     * @since 5.30
+     */
     void copyJobStarted(KIO::CopyJob *job);
 
     /**
      * Signals that the popup menu is about to be shown.
      * Applications can use the information provided about the dropped URLs
-     * (e.g. the mimetype) to decide whether to call setApplicationActions.
+     * (e.g. the MIME type) to decide whether to call setApplicationActions.
      * @param itemProps properties of the dropped items
      */
     void popupMenuAboutToShow(const KFileItemListProperties &itemProps);
@@ -104,11 +115,10 @@ protected Q_SLOTS:
     void slotResult(KJob *job) override;
 
 protected:
-    DropJob(DropJobPrivate &dd);
+    KIOWIDGETS_NO_EXPORT explicit DropJob(DropJobPrivate &dd);
 
 private:
     Q_DECLARE_PRIVATE(DropJob)
-    Q_PRIVATE_SLOT(d_func(), void slotStart())
 };
 
 /**
@@ -152,7 +162,10 @@ KIOWIDGETS_EXPORT DropJob *drop(const QDropEvent *dropEvent, const QUrl &destUrl
  *
  * @since 5.67
  */
-KIOWIDGETS_EXPORT DropJob *drop(const QDropEvent *dropEvent, const QUrl &destUrl, DropJobFlags dropjobFlags, JobFlags flags = DefaultFlags); // TODO KF6: merge with DropJobFlags dropjobFlag = DropJobDefaultFlags
+KIOWIDGETS_EXPORT DropJob *drop(const QDropEvent *dropEvent,
+                                const QUrl &destUrl,
+                                DropJobFlags dropjobFlags,
+                                JobFlags flags = DefaultFlags); // TODO KF6: merge with DropJobFlags dropjobFlag = DropJobDefaultFlags
 
 }
 
