@@ -41,7 +41,12 @@ struct Margin {
 class Q_DECL_HIDDEN KFileItemDelegate::Private
 {
 public:
-    enum MarginType { ItemMargin = 0, TextMargin, IconMargin, NMargins };
+    enum MarginType {
+        ItemMargin = 0,
+        TextMargin,
+        IconMargin,
+        NMargins
+    };
 
     explicit Private(KFileItemDelegate *parent);
     ~Private()
@@ -96,6 +101,7 @@ public:
     void gotNewIcon(const QModelIndex &index);
 
     void paintJobTransfers(QPainter *painter, const qreal &jobAnimationAngle, const QPoint &iconPos, const QStyleOptionViewItem &opt);
+    int scaledEmblemSize(int iconSize) const;
 
 public:
     KFileItemDelegate::InformationList informationList;
@@ -107,6 +113,7 @@ public:
     QTextOption::WrapMode wrapMode;
     bool jobTransfersVisible;
     QIcon downArrowIcon;
+    QRect emblemRect;
 
 private:
     KIO::DelegateAnimationHandler *animationHandler;
@@ -123,6 +130,7 @@ KFileItemDelegate::Private::Private(KFileItemDelegate *parent)
     , showToolTipWhenElided(true)
     , wrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere)
     , jobTransfersVisible(false)
+    , emblemRect(QRect())
     , animationHandler(new KIO::DelegateAnimationHandler(parent))
     , activeMargins(nullptr)
 {
@@ -197,7 +205,7 @@ QString KFileItemDelegate::Private::itemSize(const QModelIndex &index, const KFi
 
     // Return the number of items in the directory
     const QVariant value = index.data(KDirModel::ChildCountRole);
-    const int count = value.type() == QVariant::Int ? value.toInt() : KDirModel::ChildCountUnknown;
+    const int count = value.typeId() == QMetaType::Int ? value.toInt() : KDirModel::ChildCountUnknown;
 
     if (count == KDirModel::ChildCountUnknown) {
         // was: i18nc("Items in a folder", "? items");
@@ -472,11 +480,11 @@ QBrush KFileItemDelegate::Private::brush(const QVariant &value, const QStyleOpti
     if (value.userType() == qMetaTypeId<KStatefulBrush>()) {
         return qvariant_cast<KStatefulBrush>(value).brush(option.palette);
     }
-    switch (value.type()) {
-    case QVariant::Color:
+    switch (value.typeId()) {
+    case QMetaType::QColor:
         return QBrush(qvariant_cast<QColor>(value));
 
-    case QVariant::Brush:
+    case QMetaType::QBrush:
         return qvariant_cast<QBrush>(value);
 
     default:
@@ -518,16 +526,9 @@ bool KFileItemDelegate::Private::isListView(const QStyleOptionViewItem &option)
 
 QPixmap KFileItemDelegate::Private::applyHoverEffect(const QPixmap &icon) const
 {
-    KIconEffect *effect = KIconLoader::global()->iconEffect();
-
-    // Note that in KIconLoader terminology, active = hover.
-    // ### We're assuming that the icon group is desktop/filemanager, since this
-    //     is KFileItemDelegate.
-    if (effect->hasEffect(KIconLoader::Desktop, KIconLoader::ActiveState)) {
-        return effect->apply(icon, KIconLoader::Desktop, KIconLoader::ActiveState);
-    }
-
-    return icon;
+    QPixmap result = icon;
+    KIconEffect::toActive(result);
+    return result;
 }
 
 void KFileItemDelegate::Private::gotNewIcon(const QModelIndex &index)
@@ -826,7 +827,7 @@ KFileItemDelegate::KFileItemDelegate(QObject *parent)
     d->setHorizontalMargin(Private::ItemMargin, 0, 0);
 
     // Margins for vertical mode (icon views)
-    d->setVerticalMargin(Private::TextMargin, 6, 2);
+    d->setVerticalMargin(Private::TextMargin, 6, 0);
     d->setVerticalMargin(Private::IconMargin, focusHMargin, focusVMargin);
     d->setVerticalMargin(Private::ItemMargin, 0, 0);
 
@@ -872,8 +873,8 @@ QString KFileItemDelegate::Private::display(const QModelIndex &index) const
 {
     const QVariant value = index.data(Qt::DisplayRole);
 
-    switch (value.type()) {
-    case QVariant::String: {
+    switch (value.typeId()) {
+    case QMetaType::QString: {
         if (index.column() == KDirModel::Size) {
             return itemSize(index, fileItem(index));
         } else {
@@ -882,11 +883,11 @@ QString KFileItemDelegate::Private::display(const QModelIndex &index) const
         }
     }
 
-    case QVariant::Double:
+    case QMetaType::Double:
         return QLocale().toString(value.toDouble(), 'f');
 
-    case QVariant::Int:
-    case QVariant::UInt:
+    case QMetaType::Int:
+    case QMetaType::UInt:
         return QLocale().toString(value.toInt());
 
     default:
@@ -999,16 +1000,16 @@ QIcon KFileItemDelegate::Private::decoration(const QStyleOptionViewItem &option,
     const QVariant value = index.data(Qt::DecorationRole);
     QIcon icon;
 
-    switch (value.type()) {
-    case QVariant::Icon:
+    switch (value.typeId()) {
+    case QMetaType::QIcon:
         icon = qvariant_cast<QIcon>(value);
         break;
 
-    case QVariant::Pixmap:
+    case QMetaType::QPixmap:
         icon.addPixmap(qvariant_cast<QPixmap>(value));
         break;
 
-    case QVariant::Color: {
+    case QMetaType::QColor: {
         QPixmap pixmap(option.decorationSize);
         pixmap.fill(qvariant_cast<QColor>(value));
         icon.addPixmap(pixmap);
@@ -1155,6 +1156,7 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     QPixmap icon = opt.icon.pixmap(opt.decorationSize, iconMode, iconState);
 
     const KFileItem fileItem = d->fileItem(index);
+    const bool isDir = fileItem.isDir();
     if (fileItem.isHidden()) {
         KIconEffect::semiTransparent(icon);
     }
@@ -1178,6 +1180,7 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
                     pixmap = d->transition(fadeFromPixmap, pixmap, state->fadeProgress());
                 }
                 painter->drawPixmap(option.rect.topLeft(), pixmap);
+                drawSelectionEmblem(option, painter, index);
                 if (d->jobTransfersVisible && index.column() == 0) {
                     if (index.data(KDirModel::HasJobRole).toBool()) {
                         d->paintJobTransfers(painter, state->jobAnimationAngle(), iconPos, opt);
@@ -1238,6 +1241,19 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     int focusVMargin = style->pixelMetric(QStyle::PM_FocusFrameVMargin);
     QRect focusRect = textBoundingRect.adjusted(-focusHMargin, -focusVMargin, +focusHMargin, +focusVMargin);
 
+    auto drawBackground = [textBoundingRect, iconPos, style](QPainter &painter, QStyleOptionViewItem &opt) {
+        if (opt.decorationPosition != QStyleOptionViewItem::Position::Top) {
+            style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &painter, opt.widget);
+        } else {
+            auto oldrect = opt.rect;
+            opt.rect = textBoundingRect;
+            style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &painter, opt.widget);
+            opt.rect = QRect(iconPos, opt.decorationSize);
+            style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &painter, opt.widget);
+            opt.rect = oldrect;
+        }
+    };
+
     // Create a new cached rendering of a hovered and an unhovered item.
     // We don't create a new cache for a fully hovered item, since we don't
     // know yet if a hover out animation will be run.
@@ -1251,8 +1267,9 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
         p.begin(&cache->regular);
         p.translate(-option.rect.topLeft());
         p.setRenderHint(QPainter::Antialiasing);
-        style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &p, opt.widget);
+        drawBackground(p, opt);
         p.drawPixmap(iconPos, icon);
+        drawSelectionEmblem(option, painter, index);
         d->drawTextItems(&p, labelLayout, labelColor, infoLayout, infoColor, textBoundingRect);
         d->drawFocusRect(&p, opt, focusRect);
         p.end();
@@ -1263,8 +1280,9 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
         p.begin(&cache->hover);
         p.translate(-option.rect.topLeft());
         p.setRenderHint(QPainter::Antialiasing);
-        style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &p, opt.widget);
+        drawBackground(p, opt);
         p.drawPixmap(iconPos, icon);
+        drawSelectionEmblem(option, painter, index);
         d->drawTextItems(&p, labelLayout, labelColor, infoLayout, infoColor, textBoundingRect);
         d->drawFocusRect(&p, opt, focusRect);
         p.end();
@@ -1284,6 +1302,7 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
         }
 
         painter->drawPixmap(option.rect.topLeft(), pixmap);
+        drawSelectionEmblem(option, painter, index);
         painter->setRenderHint(QPainter::Antialiasing);
         if (d->jobTransfersVisible && index.column() == 0) {
             if (index.data(KDirModel::HasJobRole).toBool()) {
@@ -1303,11 +1322,12 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
         icon = d->applyHoverEffect(icon);
     }
 
-    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);
+    drawBackground(*painter, opt);
     painter->drawPixmap(iconPos, icon);
 
     d->drawTextItems(painter, labelLayout, labelColor, infoLayout, infoColor, textBoundingRect);
     d->drawFocusRect(painter, opt, focusRect);
+    drawSelectionEmblem(option, painter, index);
 
     if (d->jobTransfersVisible && index.column() == 0 && state) {
         if (index.data(KDirModel::HasJobRole).toBool()) {
@@ -1317,6 +1337,33 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     painter->restore();
 }
 
+void KFileItemDelegate::drawSelectionEmblem(QStyleOptionViewItem option, QPainter *painter, const QModelIndex &index) const
+{
+    if (index.column() != 0 || !qApp->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick)) {
+        return;
+    }
+    const auto state = option.state;
+    if ((state & QStyle::State_MouseOver && !fileItem(index).isDir()) || (state & QStyle::State_Selected)) {
+        const QString selectionEmblem = state & QStyle::State_Selected ? QStringLiteral("emblem-remove") : QStringLiteral("emblem-added");
+        const auto emblem = QIcon::fromTheme(selectionEmblem).pixmap(d->emblemRect.size(), state & QStyle::State_MouseOver ? QIcon::Active : QIcon::Disabled);
+
+        painter->drawPixmap(d->emblemRect.topLeft(), emblem);
+    }
+}
+
+int KFileItemDelegate::Private::scaledEmblemSize(int iconSize) const
+{
+    if (iconSize <= KIconLoader::SizeSmallMedium) {
+        return KIconLoader::SizeSmall;
+    } else if (iconSize <= KIconLoader::SizeHuge) {
+        return KIconLoader::SizeSmallMedium;
+    } else if (iconSize <= KIconLoader::SizeEnormous) {
+        return KIconLoader::SizeMedium;
+    }
+
+    return KIconLoader::SizeHuge;
+}
+
 QWidget *KFileItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
     QStyleOptionViewItem opt(option);
@@ -1553,4 +1600,26 @@ bool KFileItemDelegate::eventFilter(QObject *object, QEvent *event)
     } // switch (event->type())
 }
 
+void KFileItemDelegate::setSelectionEmblemRect(QRect rect, int iconSize)
+{
+    const auto emblemSize = d->scaledEmblemSize(iconSize);
+
+    // With small icons, try to center the emblem on top of the icon
+    if (iconSize <= KIconLoader::SizeSmallMedium) {
+        d->emblemRect = QRect(rect.topLeft().x() + emblemSize / 4, rect.topLeft().y() + emblemSize / 4, emblemSize, emblemSize);
+    } else {
+        d->emblemRect = QRect(rect.topLeft().x(), rect.topLeft().y(), emblemSize, emblemSize);
+    }
+}
+
+QRect KFileItemDelegate::selectionEmblemRect() const
+{
+    return d->emblemRect;
+}
+
+KFileItem KFileItemDelegate::fileItem(const QModelIndex &index) const
+{
+    return d->fileItem(index);
+}
+
 #include "moc_kfileitemdelegate.cpp"
